package main

import (
	"database/sql"
	"errors"
	"log"

	_ "github.com/go-sql-driver/mysql"
)

//func transfer(db *sql.DB, id int, money float64) error {
func transfer(tx *sql.Tx, id int, money float64) error {
	if money < 0 {
		var currentMony float64
		err := tx.QueryRow(`select money from test where id = ?;`, id).Scan(&currentMony)
		if err != nil {
			return err
		}

		if currentMony < -money {
			return errors.New("not enough money")
		}
	}

	_, err := tx.Exec(`update test set money = money + ? where id = ?`, money, id)
	if err != nil {
		return err
	}

	return nil
}

func main() {
	// 格式有对应的驱动定义
	// user:password@tcp(host:port)/database?charset=utf8mb4&loc=PRC&parseTime=true
	dataSourceName := "root:root@tcp(100.80.0.128:3306)/golangdev?charset=utf8mb4&loc=PRC&parseTime=true"
	db, err := sql.Open("mysql", dataSourceName)
	if err != nil {
		log.Fatal(err)
		return
	}
	// 测试连接状态
	log.Println("ping", db.Ping())

	// 执行
	// 创建数据库
	_, err = db.Exec(`create table if not exists test(
	    id bigint primary key auto_increment,
	    name varchar(32) not null comment "user name",
    	money decimal(21,5) not null default 0
	) engine=innodb default charset utf8mb4;`)
	if err != nil {
		log.Println("create table err", err)
	}

	// 初始化用户
	_, _ = db.Exec(`insert into test(name, money) value ("xie", 1000);`)
	_, _ = db.Exec(`insert into test(name, money) value ("xian", 500);`)

	// 转账，需要保证同时成功或失败
	//var money float64 = 200
	var money float64 = 1000
	tx, err := db.Begin()
	err1 := transfer(tx, 1, money)
	if err1 != nil {
		log.Println("tack in money err", err1.Error())
	}
	err2 := transfer(tx, 2, -money)
	if err2 != nil {
		log.Println("take out money err", err2.Error())
	}

	if err1 == nil && err2 == nil {
		// 提交事务
		tx.Commit()
	} else {
		// 回滚数据
		tx.Rollback()
	}
}
